@@ -9,37 +9,22 @@ use Coro;
 use Coro::AnyEvent qw(sleep);
 use Coro::Channel;
 use Coro::ProcessPool::Process;
-use Sys::Info;
+use Coro::ProcessPool::Util qw(cpu_count);
 
-our $VERSION = '0.12_02';
+our $VERSION = '0.14_01';
 
 if ($^O eq 'MSWin32') {
     die 'MSWin32 is not supported';
 }
 
-our $CPU_COUNT = _cpu_count();
-
-use fields qw(
-    max_procs
-    num_procs
-    max_reqs
-    procs
-);
-
 sub new {
     my ($class, %param) = @_;
-    my $self = fields::new($class);
-    $self->{max_procs} = $param{max_procs} || $CPU_COUNT;
-    $self->{max_reqs}  = $param{max_reqs}  || 0;
-    $self->{num_procs} = 0;
-    $self->{procs}     = Coro::Channel->new();
-    return $self;
-}
-
-sub _cpu_count {
-    my $info = Sys::Info->new();
-    my $cpu  = $info->device('CPU');
-    return $cpu->count || 4; # default to 4 if Sys::Info fails
+    return bless {
+        max_procs => $param{max_procs} || cpu_count,
+        max_reqs  => $param{max_reqs}  || 0,
+        num_procs => 0,
+        procs     => Coro::Channel->new(),
+    }, $class;
 }
 
 sub shutdown {
@@ -107,7 +92,7 @@ sub checkout_proc {
 
 sub process {
     my ($self, $f, $args, $timeout) = @_;
-    ref $f eq 'CODE' || croak 'expected CODE ref to execute';
+    defined $f || croak 'expected CODE ref or task class (string) to execute';
     $args ||= [];
     ref $args eq 'ARRAY' || croak 'expected ARRAY ref of arguments';
 
@@ -193,6 +178,9 @@ Coro::ProcessPool - an asynchronous process pool
         print "$i = " . $deferred{$i}->() . "\n";
     }
 
+    # Use a "task class", implementing 'new' and 'run'
+    my $result = $pool->process('Task::Doubler', 21);
+
     $pool->shutdown;
 
 =head1 DESCRIPTION
@@ -230,6 +218,11 @@ Processes code ref C<$f> in a child process from the pool. If C<$args> is
 provided, it is an array ref of arguments that will be passed to C<$f>. Returns
 the result of calling C<$f->(@$args)>.
 
+Alternately, C<$f> may be the name of a class implementing the methods C<new>
+and C<run>, in which case the result is equivalent to calling
+C<$f->new(@$args)->run()>. Note that the include path for worker processes is
+identical to that of the calling process.
+
 This call will yield until the results become available. If all processes are
 busy, this method will block until one becomes available. Processes are spawned
 as needed, up to C<max_procs>, from this method. Also note that the use of
@@ -342,6 +335,12 @@ Then, in your caller:
         Bar::dostuff();
     });
 
+=head2 If it's a problem...
+
+Use the task class method if the loading requirements are causing headaches:
+
+    my $result = $pool->process('Task::Class', [@args]);
+
 =head1 COMPATIBILITY
 
 C<Coro::ProcessPool> will likely break on Win32 due to missing support for
